﻿define([
    'jquery',
    'underscore',
    'nico/drop',
    'nico/observable',
    'nico/Designer/dropPersistence',
    'nico/dropFactory',
    'widgets/templateProvider',
    './templateTraverser',
    'nico/Designer/propertyEditors'
], function ($, _, Drop, observable, dropPersistence, dropFactory, templateProvider, TemplateTraverser, propertyEditors) {
    'use strict';

    function toCamelCase(value) {
        return value
            .replace(/(?:^\w|[A-Z]|\b\w)/g, function (letter, index) {
                return index == 0 ? letter.toLowerCase() : letter.toUpperCase();
            })
            .replace(/-/g, '');
    }

    function WidgetDesign(data, $element) {
        Drop.call(this, $element, data);
        this.id = this.data.id;
        this.propertyEditors = this.createPropertyEditors();

        this.renderedPromise = $.Deferred();
    };

    Drop.extend(WidgetDesign, Drop);

    WidgetDesign.prototype.environment = 'designer';

    WidgetDesign.prototype.promise = function () {
        return this.renderedPromise.promise();
    };

    WidgetDesign.prototype.preRender = function () {
        this.fillTemplate();
    };

    WidgetDesign.prototype.fillTemplate = function () {
        var self = this;
        var dropNodes = this.getDropNodes();
        _.each(dropNodes, function ($node) {
            var naam = $node.data('naam');
            var existingDrop = self.findDropByNaam(naam);

            if (existingDrop) {
                self
                    .addExistingDropToNode($node, existingDrop)
                    .done(function () {
                        self.renderDropsInPool(dropNodes);
                    });
            } else if (self.allowsMultipleDrops($node)) {
                var data = {
                    positioning: 'relative',
                    naam: naam,
                    nestedDropType: $node.data('type') ? $node.data('type') : 'widget',
                    conditional: self.isConditional($node)
                };
                self
                    .addNewDropToNode('dropContainer', data, $node)
                    .done(function(drop) {
                        dropPersistence.saveDrop(drop);
                        self.renderDropsInPool(dropNodes);
                    });
            } else {
                if (!$node.data('type'))
                    throw new Error('The node with name "' + naam + '" is missing a data-type attribute');

                var data = {
                    positioning: 'relative',
                    naam: naam,
                    conditional: self.isConditional($node)
                };

                _.chain($node.get(0).attributes)
                    .filter(function (attribute) {
                        return attribute.name.indexOf('data-default-') === 0;
                    })
                    .each(function (attribute) {
                        var propertyName = attribute.name.substring('data-default-'.length);
                        data[toCamelCase(propertyName)] = attribute.value;
                    })
                    .value();

                self
                    .addNewDropToNode($node.data('type'), data, $node)
                    .done(function () {
                        self.renderDropsInPool(dropNodes);
                    });
            }
        });
    };

    WidgetDesign.prototype.renderDropsInPool = function (dropNodes) {
        if (this.pool.length === dropNodes.length) {
            var promises = [];
            _.each(this.pool, function (drop) {
                drop.render();

                if (_.isFunction(drop.promise))
                    promises.push(drop.promise());
            });

            if (promises.length === 0) {
                this.renderedPromise.resolve();
            } else {
                var self = this;
                $.when
                    .apply($, promises)
                    .done(function () {
                        self.renderedPromise.resolve();
                    });
            }
        }
    };

    WidgetDesign.prototype.getDropNodes = function () {
        var templateTraverser = new TemplateTraverser(this.$element);
        return templateTraverser.traverse();
    };

    WidgetDesign.prototype.allowsMultipleDrops = function ($node) {
        return typeof $node.attr('data-many') !== 'undefined';
    };

    WidgetDesign.prototype.isConditional = function ($node) {
        return typeof $node.attr('data-conditional') !== 'undefined';
    };

    WidgetDesign.prototype.addExistingDropToNode = function ($node, data) {
        var self = this;

        // note: in some cases this will allow us to change the type of the drop
        // in the template without having to write a migration
        data.dropRuntimeData.type = this.getExpectedTypeOfDrop($node);
        data.dropRuntimeData.conditional = this.isConditional($node);

        return dropFactory
            .restoreDrop(this.environment, 'buildingBlocks', data, $node)
            .done(function (drop) {
                self.addDrop(drop);
                drop.addSaveDataDecorator(function (data) {
                    self.addChildrenSaveData(data);
                });
            });
    };

    WidgetDesign.prototype.getExpectedTypeOfDrop = function ($node) {
        if (this.allowsMultipleDrops($node))
            return 'dropContainer';
        return $node.data('type');
    };

    WidgetDesign.prototype.addNewDropToNode = function (dropType, data, $node) {
        var self = this;
        return dropFactory
            .createDrop(this.environment, 'buildingBlocks', dropType, data, $node)
            .done(function(drop) {
                self.addDrop(drop);
                drop.addSaveDataDecorator(function(data) {
                    self.addChildrenSaveData(data);
                });
            });
    };

    WidgetDesign.prototype.addSaveData = function (data) {
        data.conditie = this.data.conditie;
    };

    WidgetDesign.prototype.addChildrenSaveData = function (data) {
        this.decorateSaveData(data);
        data.path.push(this.id);
    };
    
    WidgetDesign.prototype.findDropByNaam = function (naam) {
        return _.find(this.data.drops, function (dropData) {
            return dropData.naam === naam;
        });
    };

    WidgetDesign.prototype.createPropertyEditors = function () {
        var self = this;
        return [

            propertyEditors.createTextArea('Conditie')
                .withGet(function () {
                    return self.data.conditie;
                })
                .withSet(function (value) {
                    self.data.conditie = value;
                })
                .onlyShowWhen(function() {
                    return self.data.templateName || self.data.conditional;
                })
                .withDataDecorator('conditie')

        ];
    };

    WidgetDesign.prototype.getNestedDrops = function () {
        return this.pool;
    };

    WidgetDesign.prototype.createOperationRoute = 'addWidgetDrop';
    WidgetDesign.prototype.updateOperationRoute = 'updateWidgetDrop';

    return WidgetDesign;
});