define([
    'jquery',
    'scrollz',
    'nico/dispatcher',
    'underscore',
    'nico/cardFront'
], function ($, scrollz, Dispatcher, _, CardFront) {
    'use strict';

    var Feed = function ($element) {
        this.$element = $element;
        this.$feed = $('<div />')
            .appendTo(this.$element);
        this.cards = [];
        this.loading = false;
    };

    Feed.prototype.dispatcher = new Dispatcher();

    Feed.prototype.render = function (feed) {
        this.cards = this.createFeedItems(feed.items);
        this.mostRecentMessageNumber = feed.mostRecentMessageNumber;
        this.oldestMessageNumber = feed.oldestMessageNumber;

        this.$feed
            .append(_.pluck(this.cards, '$element'));

        var self = this;
        this.$element
            .height(this.$element.parent().height())
            .scrollz({
                pull: true,
                emulateTouchEvents: true,
                bottomDetectionOffset: 0
            })
            .bind('pulled', function () {
                if(!self.loading)
                    self.refreshFeed();
            })
            .bind('bottomreached', function () {
                if (!self.loading)
                    self.loadNextCards();
            });
    };

    Feed.prototype.createFeedItems = function (cards) {
        var self = this;
        return _.map(cards, function (card) {
            var cardFront = new CardFront(self, card.id, card.icon);
            cardFront.showContent(card.content);
            return cardFront;
        });
    };

    Feed.prototype.refreshFeed = function () {
        var self = this;

        this.loading = true;

        this.dispatcher
            .post('refreshFeed', {
                lastDeliveredMessageNumber: this.mostRecentMessageNumber
            })
            .done(function (result) {
                self.loading = false;

                if (result.feed.items.length !== 0)
                    self.addNewerMessagesToTheTop(result);

                self.$element.scrollz('hidePullHeader');
            });
    };

    Feed.prototype.addNewerMessagesToTheTop = function (result) {
        var newCards = this.createFeedItems(result.feed.items);

        if (result.hasMoreItemsThanPageSize) {
            _.each(this.cards, function (card) {
                card.destroy();
            });
            this.cards = newCards;
            this.oldestMessageNumber = result.feed.oldestMessageNumber;
        } else {
            this.cards = this.cards.concat(newCards);
        }

        this.mostRecentMessageNumber = result.feed.mostRecentMessageNumber;
        this.$feed.prepend(_.pluck(newCards, '$element'));
    };

    Feed.prototype.loadNextCards = function () {
        var self = this;

        this.loading = true;
        this.dispatcher
            .post('getNext', {
                lastDeliveredMessageNumber: this.oldestMessageNumber
            })
            .done(function (result) {
                self.loading = false;
                self.addOlderMessagesToTheBottom(result);
            });
    };

    Feed.prototype.addOlderMessagesToTheBottom = function(result) {
        if (result.feed.items.length === 0)
            return;

        var newCards = this.createFeedItems(result.feed.items);
        this.cards = this.cards.concat(newCards);
        this.oldestMessageNumber = result.feed.oldestMessageNumber;

        this.$feed.append(_.pluck(newCards, '$element'));
    };

    Feed.prototype.reloadCards = function () {
        var cards = _.map(this.cards, function(card) {
            return {
                cardId: card.cardId,
                contentName: card.contentName
            };
        });

        var self = this;
        this.dispatcher
            .post('reloadCards', {
                cards: cards
            })
            .done(function (result) {
                _.each(self.cards, function (card) {
                    var reloadedCard = _.findWhere(result.cards, { id: card.cardId });
                    card.replaceContent(reloadedCard.content);
                });
            });
    };

    return Feed;
});