﻿// todo this file is shared between the web and mobile site
define([
    'underscore',
    'jquery',
    'nico/oop'
], function (_, $, oop) {
    'use strict';

    var defaults = function () {
        return {
            positioning: 'absolute',
            position: { x: 0, y: 0 },
            size: { x: 100, y: 20 },
            zIndex: 1,
            autoSize: false,
            anchor: {
                top: null,
                right: null,
                bottom: null,
                left: null,
                centered: false
            }
        };
    };

    var Drop = function ($element, options) {
        this.data = $.extend(true, {}, defaults(), options);
        this.$element = $element
            .css('visibility', 'inherit');
        this.pool = [];
    };

    Drop.prototype.parent = null;

    oop.makeExtendable(Drop);

    Drop.prototype.environment = null;

    Drop.prototype.render = function ($element) {
        var self = this;

        self.preRender();

        self.$element.appendTo($element);

        _.forEach(self.pool, function (drop) {
            drop.render(self.$element);
        });

        self.postRender();

        return this;
    };

    Drop.prototype.preRender = $.noop;

    Drop.prototype.postRender = $.noop;

    Drop.prototype.error = function (errorArgs) {
        this.onError(errorArgs);

        _.forEach(this.pool, function (drop) {
            drop.error(errorArgs);
        });
    };

    Drop.prototype.onError = $.noop;

    Drop.prototype.focus = function () {
        this.$element.focus();
    };

    Drop.prototype.show = function () {
        this.$element.css('visibility', 'inherit');
        if (this.$required) {
            this.$required.css('visibility', 'inherit');
        }

        this.shown();
    };

    Drop.prototype.hide = function () {
        this.$element.css('visibility', 'hidden');
        if (this.$required) {
            this.$required.css('visibility', 'hidden');
        }

        this.hidden();
    };

    Drop.prototype.shown = function () {
        if (this.$element.css('visibility') === 'hidden')
            return;

        this.afterShown();

        _.forEach(this.pool, function (drop) {
            drop.shown();
        });
    };

    Drop.prototype.afterShown = $.noop;

    Drop.prototype.hidden = function () {
        if (this.$element.css('visibility') !== 'hidden')
            return;

        this.afterHidden();

        _.forEach(this.pool, function (drop) {
            drop.hidden();
        });
    };

    Drop.prototype.afterHidden = $.noop;

    Drop.prototype.addSaveDataDecorator = $.noop;

    Drop.prototype.applyPosition = function () {
        if (this.data.positioning === 'relative')
            return;

        var css = this.getCssPositioning();
        this.$element.css(css);
    };

    Drop.prototype.getCssPositioning = function () {
        var data = this.data;

        if (data.autoSize === true) {
            return {
                position: 'absolute',
                top: data.position.y + 'px',
                left: data.position.x + 'px',
                zIndex: data.zIndex,
                width: 'auto',
                height: 'auto'
            };
        }

        var css = {
            position: 'absolute',
            zIndex: data.zIndex
        };

        if (data.anchor.bottom && !data.anchor.top) {
            css.top = 'auto';
            css.bottom = data.anchor.bottom + 'px';
            css.height = data.size.y + 'px';
        } else if (data.anchor.bottom !== null && data.anchor.top !== null) {
            css.top = data.anchor.top + 'px';
            css.bottom = data.anchor.bottom + 'px';
            css.height = 'auto';
            css.minHeight = '1px';
        } else {
            css.top = (data.anchor.top || data.position.y) + 'px';
            css.bottom = 'auto';
            css.height = data.size.y + 'px';
        }

        if (data.anchor.right !== null && data.anchor.left === null) {
            css.left = 'auto';
            css.right = data.anchor.right + 'px';
            css.width = data.size.x + 'px';
        } else if (data.anchor.right !== null && data.anchor.left !== null) {
            css.left = data.anchor.left + 'px';
            css.right = data.anchor.right + 'px';
            css.width = 'auto';
            css.minWidth = '1px';
        } else {
            css.left = (data.anchor.left || data.position.x) + 'px';
            css.right = 'auto';
            css.width = data.size.x + 'px';
        }

        if (data.anchor.centered === true) {
            return {
                width: data.size.x + 'px',
                left: '50%',
                marginLeft: -1 * (data.size.x / 2) + 'px'
            };
        }

        return css;
    };

    Drop.prototype.addDrop = function (drop) {
        this.pool.push(drop);
        drop.parent = this;
        return this;
    };

    Drop.prototype.removeDrop = function (drop) {
        var index = this.pool.indexOf(drop);
        this.pool.splice(index, 1);
        drop.parent = null;
        return this;
    };

    Drop.prototype.removeAllDrops = function () {
        var self = this;
        _.each(this.pool.slice(0), function (drop) {
            self.removeDrop(drop);
        });
    };

    Drop.prototype.getRootDrop = function () {
        if (this.parent)
            return this.parent.getRootDrop();
        return this;
    };

    return Drop;
});